
#include <stdio.h>
#include <math.h>

#include "coords.h"
#include "tr.h"
#include "db.h"
#include "stock.h"

static void
usage ()
{
    cbanner ( "$Date: 2002/05/24 14:44:35 $", 
	      "dbin phi theta dbout", 
	      "Author", 
	      "Company/University", 
	      "email@domain" ) ;
    exit(1) ;
}

int
main (int argc, char **argv)
{
    Dbptr           db, dbout, tr;
    char           *database, *output;
    char           *newchan[3] = { "u", "v", "w" } ;
    double	    phi, theta ;
    int             result ;

    elog_init ( argc, argv ) ; 
    if ( argc-optind != 4) { 
	usage() ; 
    }

    database = argv[optind++];
    if ( dbopen(database, "r", &db) ) { 
        die ( 0, "Can't open database %s", database ) ;
    }
    db = dblookup ( db, 0, "wfdisc", 0, 0 ) ;

    phi = atof(argv[optind++]) ; 
    theta = atof(argv[optind++]) ; 

    output = argv[optind++] ; 
    if ( dbopen(output, "r+", &dbout) < 0 ) { 
        die (0, "Can't open output database '%s'", output ) ; 
    }
    dbout = dblookup ( dbout, 0, "wfdisc", 0, 0 ) ;

    tr = dbinvalid() ;
    if ( trload_css ( db, "time", "time+10", &tr, 0, 0 ) < 0 ) { 
        die ( 0, "trload_css fails" ) ;
    }

    if ( trapply_calib ( tr ) ) {
        die ( 0, "trapply_calib fails" ) ;
    }

    result = trrotate ( tr, phi, theta, newchan ) ;
    if ( result < 0 ) {
        die ( 0, "rotate_to_standard fails" ) ; 
    } else if ( result > 0 ) { 
        complain ( 0, "some channels not rotated." ) ;
    }

    trdisp ( tr, "Rotation Example" ) ; 

    result = trsave_wf ( tr, dbout, "sd", 0, 0 ) ;
    if ( result < 0 ) {
	die ( 0, "trsave_wf fails\n" ) ;
    } else if ( result > 0 ) {
	if ( result & TR_CLIPPED ) {
	    complain ( 0, "some waveforms are clipped" ) ; 
	} 
	if ( result & TR_TRUNCATED ) {
	    complain ( 0, "Some waveforms lost resolution due to truncation to output integers" ) ;
	}
    }

    return result;
}
