#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>

#include "Pkt.h"
#include "coords.h"
#include "stock.h"

static void
usage ()
{
    cbanner ( "$Date: 2002/05/24 14:42:50 $", 
	     "",
	     "BRTT example", 
	     "BRTT", 
	     "support@brtt.com" ) ; 
    exit (1);
}

Packet         *
new_weather_packet ()
{
    Packet         *pkt;
    PktChannel     *achan;
    int             i,
                    j;
    char           *net = "NET";
    char           *sta = "STA";
    char           *loc = "";

    pkt = newPkt ();
    strcpy (pkt->parts.src_net, net);
    strcpy (pkt->parts.src_sta, sta);
    strcpy (pkt->parts.src_loc, "");
    strcpy (pkt->parts.src_suffix, "weather");
    pkt->pkttype = suffix2pkttype(pkt->parts.src_suffix);
    strcpy (pkt->parts.src_subcode, "");
    pkt->time = str2epoch ("March 8, 1995 22:30");
    pkt->nchannels = 3;
    pkt->channels = newtbl (pkt->nchannels);
    for (i = 0; i < pkt->nchannels; i++) {
	allot (PktChannel *, achan, 1);
	pushtbl(pkt->channels, achan) ;
	achan->nsamp = 1;
	allot (int *, achan->data, achan->nsamp);
	for (j = 0; j < achan->nsamp; j++) {
	    achan->data[j] = i + j;
	}
	achan->samprate = 0.1;
	achan->calib  = 3.14159e-5 ;
	achan->calper = 2.78e+2 ;
	achan->time = pkt->time;
	strcpy (achan->net, net);
	strcpy (achan->sta, sta);
	switch (i) {
	  case 0:
	    strcpy (achan->chan, "temp");
	    break;
	  case 1:
	    strcpy (achan->chan, "pressure");
	    break;
	  case 2:
	    strcpy (achan->chan, "humidity");
	    break;
	  default:
	    strcpy (achan->chan, "unknown");
	    break;
	}
	strcpy (achan->net, loc);
    }
    return pkt;
}

int
main (int argc, char **argv)
{
    int             c,
                    verbose = 0,
                    errflg = 0;
    Packet         *pkt,
                   *upkt = 0;
    char           *orbpkt;
    int             orbpkt_sz = 0,
                    nbytes;
    int             x;
    char            srcname[ORBSRCNAME_SIZE];
    double          time;


    elog_init (argc, argv);

    while ((c = getopt (argc, argv, "vV")) != -1) {
	switch (c) {

	  case 'v':
	    verbose++;
	    break;

	  case 'V':
	    usage ();
	    break;

	  case '?':
	    errflg++;
	    break;
	}
    }

    if (errflg)
	usage ();

    pkt = new_weather_packet ();
    x = stuffPkt (pkt, srcname, &time, &orbpkt, &nbytes, &orbpkt_sz);
    if (x < 0) {
	complain (0, "stuffPkt routine failed\n" ) ;
    } else {
	int orb ; 

	orb = orbopen ( "./weather.pkts", "w" ) ; 
	orbput(orb, srcname, time, orbpkt, nbytes ) ; 
	orbclose(orb) ; 

	showPkt (0, srcname, time, orbpkt, nbytes, stdout, PKT_DUMP);
	showPkt (0, srcname, time, orbpkt, nbytes, stdout, PKT_UNSTUFF);
	x = unstuffPkt (srcname, time, orbpkt, nbytes, &upkt);
	if (x < 0) {
	    complain (0, "unstuffPkt routine failed for %s\n",
		      srcname);
	} else {
	    if (memcmp (pkt, upkt, sizeof (pkt)) != 0) {
		complain (0, "packet and unstuffed packet are different");
	    }
	}
    }
    return 0;
}
