#include <stdio.h>
#include <math.h>

#include "tr.h"
#include "coords.h"
#include "db.h"
#include "stock.h"

static void
usage ()
{
    cbanner ( "$Date: 2002/05/24 14:42:48 $", 
	     "",
	     "BRTT example", 
	     "BRTT", 
	     "support@brtt.com" ) ; 
    exit (1);
}

int             Verbose = 0;

void show_result ( int result ) {
    if ( result < 0 ) { 
	complain ( 1, "trsave_wf fails" ) ;
    } else if ( result != 0 ) { 
	if ( result & TR_CLIPPED ) { 
	    complain ( 0, "data was clipped to fit into format" ) ;
	} 
	if ( result & TR_TRUNCATED ) { 
	    complain ( 0, "data was truncated to fit into format" ) ;
	} 
	if ( result & TR_GAPS ) { 
	    complain ( 0, "data was truncated to fit into format" ) ;
	} 
    }
}

int
main (int argc, char **argv)
{
    int             c,
                    errflg = 0;
    Dbptr           db, tr;
    char           *output;
    int		    n ;
    Wftype 	   *wftype  ;
    int		    result ;
    char	   *sigargs[] = { "try_wf", "-f", ".1", "-T", "tri", "-n", "20", "-a", "35000", 0 } ;
    int		    sigargc = sizeof(sigargs)/sizeof(char *) - 1 ; 

    elog_init ( argc, argv ) ; 

    while ((c = getopt (argc, argv, "v")) != -1) {
	switch (c) {

	case 'v':
	    Verbose = 1;
	    break;

	default:
	    errflg++ ;
	}
    }

    if (errflg || argc - optind != 0)
	usage ();
 
    output = "db" ;
    if ( dbopen(output, "r+", &db ) ) { 
	die ( 1, "Can't open output database %s", output ) ; 
    }
    db = dblookup (db, 0, "wfdisc", 0, 0);
    dbquery ( db, dbRECORD_COUNT, &n ) ; 
    if ( n > 0 ) { 
	if ( askyn ( "Ok to delete db.wfdisc? " ) ) { 
	    dbtruncate(db, 0) ; 
	} else { 
	    die ( 0, "Aborted: please move db.wfdisc out of the way." ) ; 
	}
    }
    
    tr = trnew (0, 0);
    tr = dblookup (tr, 0, "trace", 0, 0);
            
    if (trsignal_argv (tr, sigargc, sigargs)) {
        die (1, "Can't fake it with these parameters.");
    }

    tr.record = 0 ;
    wftype = trwftype("xx") ;
    printf ( "trying datatype %s : %s\n", wftype->datatype, wftype->desc ) ;
    result = trsave_wf ( tr, db, "xx", "wf/xx", trOVERWRITE ) ;
    show_result (result) ; 

    wftype = trwftype("s4") ;
    printf ( "trying datatype %s : %s\n", wftype->datatype, wftype->desc ) ;
    result = trsave_wf ( tr, db, "s4", "wf/s4", trOVERWRITE ) ;
    show_result (result) ; 

    trdestroy(&tr) ;
    return 0;
}
