#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>

#include "coords.h"
#include "db.h"
#include "stock.h"

static void
usage ()
{
    cbanner ( "$Date: 2002/05/24 14:42:48 $", 
	     "db[.table]", 
	     "BRTT example", 
	     "BRTT", 
	     "support@brtt.com" ) ; 
    exit (1);
}

int
main (int argc, char **argv)
{
    int             c,
                    errflg = 0;
    Dbptr           db, dbk ;
    char           *database;
    int             verbose = 0;
    double	    time, endtime ; 
    int		    nrecords ;
    Tbl		   *keys, *list ; 
    Hook	   *hook = 0 ; 

    elog_init ( argc, argv ) ; 

    while ((c = getopt (argc, argv, "vV")) != -1) {
	switch (c) {

	case 'v':
	    verbose++ ;
	    break;

	case 'V':
	    usage() ;
	    exit (0);

	default:
	    errflg++;
	    break ;
	}
    }

    if (errflg || argc-optind != 1)
	usage ();

    database = argv[optind++];
    nrecords = dbopen_database ( database, "r", &db ) ; 
    if ( nrecords < 0 ) { 
	die ( 0, "Can't open database %s", database ) ; 
    }
    if (db.table < 0 ) { 
	db = dblookup ( db, 0, "wfdisc", 0, 0 ) ; 
	dbquery ( db, dbRECORD_COUNT, &nrecords ) ; 
    }
    if ( nrecords < 1 ) { 
	die ( 0, "no records in table" ) ; 
    }

    dbk = db ; 
    dbk.record = dbSCRATCH ; 

    keys = strtbl("sta", "time::endtime", 0 ) ; 

    for ( ;; ) {
	char *sta, *answer ; 
	int i ;
	sta = ask ( "Station: " ) ; 
	if ( strlen(sta) == 0 ) { 
	    printf ( "\n" ) ;
	    break; 
	}

	answer = ask ( "Start time: " ) ; 
	while ( ! is_epoch_string ( answer, &time ) ) { 
	    complain ( 0, "not a valid time, please try again" ) ; 
	    free(answer) ; 
	    answer = ask ( "Start time: " ) ; 
	}
	free(answer) ;

	answer = ask ( "End time: " ) ; 
	while ( ! is_epoch_string ( answer, &endtime ) ) { 
	    complain ( 0, "not a valid time, please try again" ) ; 
	    free(answer) ; 
	    answer = ask ( "End time: " ) ; 
	}
	free(answer) ;
	dbputv ( dbk, 0, "sta", sta, "time", time, "endtime", endtime, 0 ) ; 
	
	db.record = dbALL ;
	nrecords = dbmatches ( dbk, db, &keys, &keys, &hook, &list ) ; 

	printf ( "%3d matches:\n", maxtbl(list) ) ; 
	for ( i=0 ; i<maxtbl(list) ; i++ ) { 
	    char rsta[32], *s1, *s2 ;
	    double rtime, rendtime ; 

	    db.record = (int) gettbl(list, i) ; 
	    dbgetv ( db, 0, "sta", rsta, "time", &rtime, "endtime", &rendtime, 0 ) ; 
	    printf ( "#%3d %-8s %s %s\n", 
	    	db.record, rsta, s1=strtime(rtime), s2=strtdelta(rendtime-rtime) ) ; 
	    free(s1) ; 
	    free(s2) ;
	}
	free(sta) ;
    }

    return 0;
}
