#include <string.h>
#include <stdio.h>
#include <ctype.h>

#include "coords.h"
#include "stock.h"
#include "db.h"

#include "someloc.h"

static void
usage (void)
{
    cbanner ( "$Date: 2002/05/24 14:42:48 $", 
	     "input-db [output-db]",
	     "BRTT example", 
	     "BRTT", 
	     "support@brtt.com" ) ; 
    exit (1);
}

int
save_solution (SL_Problem * p, Dbptr db)
{
    SL_Arrival     *arrival;
    SL_Origin      *origin;
    int             i,
                    nass,
                    ndef;
    int             ntime = 0;
    double          l2,
                    sum_sqrtime = 0.0;
    double          lddate;
    int             orid,
                    grn,
                    srn;
    double          esaz,
                    seaz,
                    delta;
    int             retcode = 0;


    nass = ndef = 0;
    for (i = 0; i < maxtbl (p->arrival_tbl); i++) {
	arrival = (SL_Arrival *) gettbl (p->arrival_tbl, i);
	nass++;
	if (strcmp (arrival->timedef, "d") == 0) {
	    ndef++;
	}
    }

    orid = dbnextid (db, "orid");
    if (orid < 1)
	die (0, "Can't write to lastid table\n");

    origin = &p->origin;
    origin->orid = orid;
    lddate = now ();

    db = dblookup (db, 0, "origin", 0, 0);
    db.record = dbaddnull (db);
    dbquery (db, dbRECORD_COUNT, &i);
    grn = grnumber (origin->lat, origin->lon);
    srn = srnumber (grn);
    if (db.record < 0
	    || dbputv (db, 0,
		       "orid", orid,
		       "lat", origin->lat,
		       "lon", origin->lon,
		       "depth", origin->depth,
		       "time", origin->time,
		       "jdate", yearday (origin->time),
		       "nass", nass,
		       "ndef", ndef,
		       "grn", grn,
		       "srn", srn,
		       "dtype", origin->dtype,
		       "algorithm", origin->algorithm,
		       "auth", p->author,
		       "lddate", lddate,
		       0)) {
	complain (0, "Couldn't add origin record to database.\n");
	retcode = -1;
    }
    db = dblookup (db, 0, "assoc", 0, 0);
    dbquery (db, dbRECORD_COUNT, &i);
    for (i = 0; i < maxtbl (p->arrival_tbl); i++) {
	arrival = (SL_Arrival *) gettbl (p->arrival_tbl, i);
	dist (rad (origin->lat), rad (origin->lon),
	      rad (arrival->lat), rad (arrival->lon),
	      &delta, &esaz);
	dist (rad (arrival->lat), rad (arrival->lon),
	      rad (origin->lat), rad (origin->lon),
	      &delta, &seaz);
	db.record = dbaddnull (db);
	if (db.record < 0 ||
		dbputv (db, 0,
			"arid", arrival->arid,
			"orid", origin->orid,
			"sta", arrival->sta,
			"phase", arrival->phase,
			"belief", arrival->belief,
			"delta", deg (delta),
			"seaz", deg (seaz),
			"esaz", deg (esaz),
			"timeres", arrival->timeres,
			"timedef", arrival->timedef,
			"azres", arrival->azres,
			"azdef", arrival->azdef,
			"slores", arrival->slores,
			"slodef", arrival->slodef,
			"emares", arrival->emares,
			"wgt", arrival->wgt,
			"vmodel", p->travel_time_model,
			0) != 0) {
	    complain (0, "Couldn't add assoc record to database.\n");
	    retcode = -1;
	}
	if (*(arrival->timedef) == 'd') {
	    ntime++;
	    sum_sqrtime += sqr (arrival->timeres);
	}
    }
    l2 = sqrt (sum_sqrtime / ntime);


    db = dblookup (db, 0, "origerr", 0, 0);
    dbquery (db, dbRECORD_COUNT, &i);
    db.record = dbaddnull (db);
    if (db.record < 0
	    || dbputv (db, "origerr",
		       "orid", origin->orid,
		       "sxx", origin->sxx,
		       "syy", origin->syy,
		       "szz", origin->szz,
		       "stt", origin->stt,
		       "sxy", origin->sxy,
		       "sxz", origin->sxz,
		       "syz", origin->syz,
		       "stx", origin->stx,
		       "sty", origin->sty,
		       "stz", origin->stz,
		       "sdobs", l2,
		       "smajax", origin->smajax,
		       "sminax", origin->sminax,
		       "strike", origin->strike,
		       "sdepth", origin->sdepth,
		       "stime", origin->stime,
		       "conf", origin->conf,
		       0) != 0) {
	complain (0, "Couldn't add origerr record to database.\n");
	retcode = -1;
    }
    return retcode;
}

SL_Arrival     *
new_arrival ()
{
    SL_Arrival     *arrival;
    allot (SL_Arrival *, arrival, 1);
    memset (arrival, 0, sizeof (arrival));
    arrival->timeres = -999.0;
    arrival->azres = -999.0;
    arrival->slores = -999.0;
    *arrival->timedef = '-';
    *arrival->azdef = '-';
    *arrival->slodef = '-';
    arrival->emares = -999.0;
    arrival->wgt = -1.0;
    arrival->timeres = -999.0;
    arrival->timeres = -999.0;
    arrival->timeres = -999.0;
    arrival->delta = -1.0;
    arrival->seaz = -999.0;
    arrival->esaz = -999.0;
    arrival->belief = 9.99;
    return arrival;
}

typedef struct Staloc { 
    double lat, lon ; 
} Staloc ; 

int
read_pf (Dbptr db, Pf * pf, SL_Problem * p)
{
    Tbl            *arrival_tbl;
    SL_Arrival     *arrival;
    int             i;
    char           *aline;
    static Arr     *sitearr = 0 ;
    Staloc	   *staloc ;

    if ( sitearr == 0 ) { 
	/* The following is naive, and presumes that either the site
	   table has no stations that have moved, or that the correct
	   positions are later in the table.  Of course, this need not
	   be the case.
       */
	int n ; 
	Staloc new, *old ;
	char sta[16] ;
	sitearr = newarr(0) ; 
	db = dblookup ( db, 0, "site", 0, 0 ) ; 
	dbquery ( db, dbRECORD_COUNT, &n ) ; 
	for ( db.record = 0 ; db.record < n ; db.record++ ) { 
	    dbgetv ( db, 0, 
		"sta", sta,
		"lat", &new.lat, 
		"lon", &new.lon, 
		0 ) ;
	    old = setarr(sitearr, sta, memdup(&new, sizeof(Staloc))) ; 
	    if ( old != 0 ) { 
		free(old) ; 
	    }
	}
    }

    p->author = pfget_string (pf, "author");
    strncpy (p->origin.author, p->author, 16);
    p->output_file = pfget_string (pf, "output_file");
    p->verbose = pfget_boolean (pf, "verbose");
    p->fix_depth = pfget_boolean (pf, "fix_depth");
    p->max_iterations = pfget_int (pf, "maximum_iterations");
    p->use_starting_location = pfget_boolean (pf, "use_starting_location");
    p->initial_latitude = pfget_double (pf, "initial_latitude");
    p->initial_longitude = pfget_double (pf, "initial_longitude");
    p->initial_depth = pfget_double (pf, "initial_depth");
    p->default_deltim = pfget_double (pf, "default_deltim");
    p->default_delaz = pfget_double (pf, "default_delaz");
    p->default_delslo = pfget_double (pf, "default_delslo");
    p->travel_time_model = pfget_string (pf, "travel_time_model");

    arrival_tbl = pfget_tbl (pf, "arrival_table");

    arrival = new_arrival ();
    for (i = 0; i < maxtbl (arrival_tbl); i++) {
	aline = gettbl (arrival_tbl, i);
	sscanf (aline, "%d %s %s %lf %lf %s %lf %lf %s %lf %lf %s",
		&arrival->arid,
		arrival->sta,
		arrival->iphase,
		&arrival->time,
		&arrival->deltim,
		arrival->timedef,
		&arrival->azimuth,
		&arrival->delaz,
		arrival->azdef,
		&arrival->slow,
		&arrival->delslo,
		arrival->slodef
	    );

	p->ndata = 0;
	if (*arrival->timedef == 'd') {
	    if (arrival->deltim >= 0) {
		arrival->deltim = sqrt (SQR (arrival->deltim) + SQR (p->default_deltim));
	    } else {
		arrival->deltim = p->default_deltim;
	    }
	    p->ndata++;
	}
	if (*arrival->azdef == 'd') {
	    if (arrival->delaz >= 0) {
		arrival->delaz = sqrt (SQR (arrival->delaz) + SQR (p->default_delaz));
	    } else {
		arrival->delaz = p->default_delaz;
	    }
	    p->ndata++;
	}
	if (*arrival->slodef == 'd') {
	    if (arrival->delslo >= 0) {
		arrival->delslo = sqrt (SQR (arrival->delslo) + SQR (p->default_delslo));
	    } else {
		arrival->delslo = p->default_delslo;
	    }
	    p->ndata++;
	}
	staloc = getarr(sitearr, arrival->sta) ; 
	if ( staloc == 0 ) { 
	    complain(0, "No information about the location of station %s\n", 
	    	arrival->sta ) ; 
	} else { 
	    arrival->lat = staloc->lat ; 
	    arrival->lon = staloc->lon ; 
	    pushtbl (p->arrival_tbl, arrival);
	}
    }
    free (arrival);
    freetbl (arrival_tbl, 0);
    return 0;
}

int
get_solution (Dbptr db, SL_Problem * p)
{
    SL_Origin      *origin;
    SL_Arrival     *arrival;
    double          x[3],
                    c[3];
    double          tmin;
    int             i;
    FILE           *file;
    int             no_solution = 1;
    char	   *s ;
    int		    sign ;

    file = fopen (p->output_file, "w");

    origin = &p->origin;
    tmin = 9e37;
    c[0] = c[1] = c[2] = 0;

    /* Here's where you do the work */
    /* This is also where you put the verbose output */

    /* Needless to say, all of the values filled in in this 
       routine are BOGUS.  They're just here as an example.
    */
    fprintf (file, "someloc $Revision: 1.2 $ \n\n");
    fprintf ( file, "%d arrivals\n", maxtbl(p->arrival_tbl)) ;
    for (i = 0; i < maxtbl (p->arrival_tbl); i++) {
	arrival = gettbl (p->arrival_tbl, i);
	fprintf ( file, "%-8s %7.3f %7.3f %s %-3s\n", 
		arrival->sta, arrival->lat, arrival->lon, 
		s=epoch2str(arrival->time, "%H:%M"), 
		arrival->iphase ) ; 
	tmin = MIN (tmin, arrival->time);
	dsphcar (arrival->lon, arrival->lat, x);
	fprintf ( file, "\t%5.3f %5.3f %5.3f\n", x[0], x[1], x[2] ) ;
	dr3add (x, c, c);
    }
    dr3norm (c);
    dcarsph (c, &origin->lon, &origin->lat);
    fprintf ( file, "\t%5.3f %5.3f %5.3f => %7.3f %7.3f\n", c[0], c[1], c[2], 
    		origin->lat, origin->lon ) ;
    origin->depth = 123.456 ;
    origin->time = tmin - 100;

    origin->sxx = 1 ; 
    origin->syy = 1 ; 
    origin->szz = 1 ; 
    origin->stt = 1 ; 
    origin->sxy = 0 ; 
    origin->sxz = 0 ; 
    origin->syz = 0 ; 
    origin->stx = 0 ; 
    origin->sty = 0 ; 
    origin->stz = 0 ; 
    origin->smajax = 180.0 ; 
    origin->sminax = 90.0 ; 
    origin->strike = 0.0 ; 
    origin->sdepth = 10.0 ; 
    origin->stime = 10.0 ; 
    origin->conf = 10.0 ; 

    no_solution = 0;

    sprintf (origin->algorithm, "%s:%s", "someloc", p->travel_time_model);

    for (i = 0; i < maxtbl (p->arrival_tbl); i++) {
	arrival = gettbl (p->arrival_tbl, i);
	strcpy(arrival->phase, arrival->iphase) ;
	sign = ((i % 2) == 0) ? 1 : -1 ;
	arrival->timeres = i * .01 * sign ;
    }

    fclose (file);
    return no_solution;
}

void
null_origin (SL_Origin * origin)
{
    origin->lat = -999.0000;
    origin->lon = -999.0000;
    origin->depth = -999.0000;
    origin->time = -9999999999.99900;

    origin->sxx = -999999999.9999;
    origin->syy = -999999999.9999;
    origin->szz = -999999999.9999;
    origin->stt = -999999999.9999;
    origin->sxy = -999999999.9999;
    origin->sxz = -999999999.9999;
    origin->syz = -999999999.9999;
    origin->stx = -999999999.9999;
    origin->sty = -999999999.9999;
    origin->stz = -999999999.9999;
    origin->smajax = -1.0;
    origin->sminax = -1.0;
    origin->strike = -1.0;
    origin->conf = 0.0;
    origin->sdepth = -1.0;
    origin->stime = -1.0;
}

SL_Problem     *
new_problem ()
{
    SL_Problem     *p;
    allot (SL_Problem *, p, 1);
    memset (p, 0, sizeof (p));
    p->arrival_tbl = inittbl (0, 0, 1, 0, sizeof (SL_Arrival));
    null_origin (&p->origin);
    return p;
}

int
locate_event (Dbptr dbin, Pf * pf, Dbptr dbout, char **errmsg, int *orid)
{
    int             retcode = 0;
    static SL_Problem *p = 0;

    if (p == 0) {
	p = new_problem ();
    } else {
	*p->errmsg = 0;
	clrtbl (p->arrival_tbl, 0);
	null_origin (&p->origin);
    }

    if (read_pf (dbin, pf, p)) {
	retcode = -1;
    } else if (get_solution (dbin, p)) {
	retcode = -1;
    } else if (save_solution (p, dbout)) {
	retcode = -1;
    }
    *orid = p->origin.orid;
    *errmsg = p->errmsg;

    return retcode;
}

int
main (int argc, char **argv)
{
    char           *pfname;
    Pf             *pf;
    char           *dbinname,
                   *dboutname;
    Dbptr           dbin,
                    dbout;
    char            aline[STRSZ];
    int             result,
                    orid;
    char           *errmsg;

    elog_init (argc, argv);
    pfname = argv[0];

    if (argc - optind < 1 || argc - optind > 2) {
	usage ();
    }
    dbinname = argv[optind++];
    if (dbopen (dbinname, "r+", &dbin) < 0) {
	die (1, "Can't open database %s\n", dbinname);
    }
    if (argc - optind > 0) {
	dboutname = argv[optind++];
	if (dbopen (dboutname, "r+", &dbout) < 0) {
	    die (1, "Can't open database %s\n", dboutname);
	}
    } else {
	dbout = dbin;
    }

    if (pfread (Program_Name, &pf) != 0)
	die (0, "Can't read parameter file %s", pfname);

    while (gets (aline)) {
	pfupdate (pfname, &pf);
	fprintf (stderr, "Starting solver...\n");
	result = locate_event (dbin, pf, dbout, &errmsg, &orid);
	if (result < 0) {
	    printf ("location_solution: no_solution : %s\n", errmsg);
	} else {
	    printf ("location_solution: new origin %d\n", orid);
	}
	fflush (stdout);
    }

    return (0);
}
