#ifndef __someloc__
#define __someloc__

typedef struct {
    char            sta[16];		/* from input pf */
    char            chan[16];		/* (arrival table) */
    double          time;
    double          lat,
                    lon;
    int             arid;
    char            iphase[9];
    double          deltim;
    double          azimuth;
    double          delaz;
    double          slow;
    double          delslo;

    char            phase[16];		/* filled in by get_solution */
    double          timeres;		/* (assoc table) */
    char            timedef[16];
    double          azres;
    char            azdef[16];
    double          slores;
    char            slodef[16];
    double          emares;
    double          wgt;
    double          belief;

    double          delta;		/* calculated by save_solution */
    double          seaz;
    double          esaz;
}               SL_Arrival;

typedef struct SL_Origin {
    char            author[16];		/* from input pf */

    double          lat;		/* filled in by get_solution */
    double          lon;		/* (origin) */
    double          depth;
    double          time;
    char            algorithm[16];
    char            dtype[2];

    double          sxx,		/* (origerr) */
                    syy,
                    szz,
                    stt,
                    sxy,
                    sxz,
                    syz,
                    stx,
                    sty,
                    stz,
                    smajax,
                    sminax,
                    strike,
                    conf,
                    sdepth,
                    stime ;

    int             orid;		/* filled in by save_solution */
    double          sdobs;

}               SL_Origin;

typedef struct SL_Problem {
    char           *author;	       /* for origin.author field */
    char           *output_file;       /* output file */
    int             verbose;	       /* non-zero => higher verbosity in
				        * output file */
    int             fix_depth;	       /* non-zero => fix the solution depth */
    int             max_iterations;    /* maximum number of iterations to
				        * allow */
    int             use_starting_location;	/* non-zero => use specified
						 * starting location,
						 * otherwise make one up */
    double          initial_latitude;  /* starting latitude */
    double          initial_longitude; /* starting longitude */
    double          initial_depth;     /* starting depth */
    double          default_deltim;    /* default uncertainty in time */
    double          default_delaz;     /* default uncertainty in azimuth */
    double          default_delslo;    /* default uncertainty in slowness */
    char           *travel_time_model; /* travel time model */
    char            errmsg[1024] ;     /* error message from solver */
    int             ndata;
    Tbl            *arrival_tbl;       /* list of SL_Arrival arrivals */
    SL_Origin       origin;
}               SL_Problem;

#endif
