#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>

#include "Pkt.h"
#include "coords.h"

static void
usage ()
{
    cbanner ("$Date: 2002/05/24 14:42:49 $",
	     "[-t dt] [-v] orb",
	     "BRTT example", 
	     "BRTT", 
	     "support@brtt.com" ) ; 
    exit (1);
}

int
main (int argc, char **argv)
{
    int             c,
                    verbose = 0,
                    errflg = 0;
    int             packetsz = 0,
                    nbytes;
    int             i = 0,
                    dt = 1;
    char           *packet;
    Packet         *pkt;
    char            srcname[ORBSRCNAME_SIZE];
    double          time,
                    t,
                    tnext;
    char           *out;
    int             orb;

    elog_init (argc, argv);

    while ((c = getopt (argc, argv, "t:vV")) != -1) {
	switch (c) {

	  case 't':
	    dt = atoi (optarg);
	    break;

	  case 'v':
	    verbose++;
	    break;

	  case 'V':
	    usage ();
	    break;

	  case '?':
	    errflg++;
	    break;
	}
    }

    if (errflg || argc - optind != 1)
	usage ();

    out = argv[optind++];
    if ((orb = orbopen (out, "w&")) < 0)
	die (0, "Can't open output '%s'\n", out);

    pkt = newPkt ();
    strcpy (pkt->parts.src_net, "BRTT");
    pkt->pkttype = suffix2pkttype ("ch");

    tnext = now ();
    for (;;) {
	t = now ();
	if (pkt->string) {
	    free (pkt->string);
	}
	pkt->string = strtime (t);
	pkt->string_size = strlen (pkt->string);

	if (stuffPkt (pkt, srcname, &time, &packet, &nbytes, &packetsz) < 0) {
	    complain (0, "stuffPkt routine failed for %s", pkt->pkttype->name);
	} else if (verbose) {
	    showPkt (i, srcname, time, packet, nbytes, stdout, PKT_DUMP);
	}
	if (orbput (orb, srcname, time, packet, nbytes) < 0) {
	    complain (0, "Couldn't send packet to %s\n", out);
	    break;
	}
	tnext = floor (t + dt);
	wait_for_time (tnext, verbose > 1);

    }
    /* NOTREACHED */

    return 0;
}
