#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>

#include "Pkt.h"
#include "coords.h"

static void
usage ()
{
    cbanner ("$Date: 2002/05/24 14:42:49 $",
	     "[-d] [-m match] [-n #packets] [-r reject] [-S state] [-v] orb [time [range]]",
	     "BRTT example", 
	     "BRTT", 
	     "support@brtt.com" ) ; 
    exit (1);
}

void
processPkt ( int pktid, char *srcname, double time, char *packet, int nbytes, int mode ) 
{
    static Packet   *unstuffed=0 ;
    int type ; 

    showPkt ( pktid, srcname, time, packet, nbytes, stdout, mode ) ;
    type = unstuffPkt (srcname, time, packet, nbytes, &unstuffed) ;
    switch (type) {

	case Pkt_wf :
	    printf ( "waveform packet: %s\n", srcname ) ; 
	    break ; 

	case Pkt_st :
	    printf ( "status packet : %s\n", srcname ) ; 
	    break ; 

	case Pkt_db :
	    printf ( "database row: %s\n", srcname ) ; 
	    break ; 

	case Pkt_pf :
	    printf ( "parameter file: %s\n", srcname ) ; 
	    break ; 

	case Pkt_cn :
	    printf ( "control messages: %s\n", srcname ) ; 
	    break ; 

	case Pkt_rw :
	    printf ( "arbitrary binary data, wrapped in orb packet: %s\n", srcname ) ; 
	    break ; 

	case Pkt_ch :
	    printf ( "arbitrary character string: %s\n", srcname ) ; 
	    break ; 

	case Pkt_tp :
	    printf ( "test packet: %s\n", srcname ) ; 
	    break ; 

	default:
	    printf ( "unrecognized packet type %d: %s\n", type, srcname ) ; 
	    break ; 
    }
}

int
main (int argc, char **argv)
{
    int 	   verbose = 0 ;
    double	    time ;
    char	   *orbname=0, *s ; 
    int             c,
                    errflg = 0;
    int		    orb ;
    int		    pktid ; 
    int		    totpkts=0, totbytes=0 ;
    double	    start_time, end_time, delta_t ;
    char	    srcname[ORBSRCNAME_SIZE] ;
    double 	    after = 0.0, quit = 0.0  ; 
    char	   *match = 0, *reject = 0 ; 		
    int		    n2read = 1e9 ;
    int		    nmatch ;
    char	   *packet=0 ; 
    int		    nbytes = 0 ;
    int		    rcode ;
    int		    bufsize=0 ;
    int		    abort = 0 ;
    int		    mode = PKT_NOSAMPLES ;
    char	   *statefile = 0 ;

    Program_Name = argv[0];
    elog_init ( argc, argv ) ; 

    while ((c = getopt (argc, argv, "dm:n:r:S:Vv")) != -1)
	switch (c) {

	case 'd':
	    mode = PKT_DUMP ;
	    verbose = 1 ;
	    break ;

	case 'm':
	    match = optarg ;
	    break ;

	case 'n':
	    n2read = atoi(optarg) ;
	    break ;

	case 'r':
	    reject = optarg ; 
	    break ;

	case 'S':
	    statefile = optarg ; 
	    break; 

	case 'V':
	    usage ();
	    break;

	case 'v':
	    verbose++ ;
	    break ;

	case '?':
	    errflg++;
	}

    if (errflg || (argc-optind < 1 || argc-optind > 3))
	usage ();

    orbname = argv[optind++] ; 
    if ( argc > optind ) {
	after = str2epoch ( argv[optind++] ) ;
        if ( argc > optind ) {
            quit = str2epoch ( argv[optind++] ) ;
            if ( quit < after )
		quit += after ;
        }
    }

    if ( (orb = orbopen ( orbname, "r&" )) < 0 )
	die ( 0, "Can't open ring buffer '%s'\n", orbname ) ; 

    if ( match ) {
	if ( (nmatch = orbselect ( orb, match )) < 0 )
	    complain ( 1, "orbselect '%s' failed\n", match ) ;
	else
	    printf ( "%d sources selected after select\n", nmatch) ; 
    }

    if ( reject ) {
	if ( (nmatch = orbreject ( orb, reject )) < 0 )
	    complain ( 1, "orbreject '%s' failed\n", match ) ;
	else
	    printf ( "%d sources selected after reject\n", nmatch) ; 
    }

    if ( after > 0 ) {
	if ((pktid = orbafter ( orb, after )) < 0) {
	    complain ( 1, "orbafter to %s failed\n", s=strtime(after) )  ;
	    free(s) ; 
	    pktid = orbtell ( orb ) ; 
	    printf ("pktid is still #%d\n", pktid ) ; 
	} else 
	    printf ("new starting pktid is #%d\n", pktid ) ; 
    }

    if ( statefile != 0 ) {
	char *s ;
	if ( exhume ( statefile, &abort, RT_MAX_DIE_SECS, 0 ) != 0 ) {
	    elog_notify ( 0, "read old state file\n" ) ;
	}
	if ( orbresurrect ( orb, &pktid, &time ) == 0 )  {
	    elog_notify ( 0, "resurrection successful: repositioned to pktid #%d @ %s\n",
		    pktid, s=strtime(time) ) ;
	    free(s) ;
	} else {
	    complain ( 0, "resurrection unsuccessful\n" ) ;
	}
    }

    rcode = orbget ( orb, ORBCURRENT, &pktid, srcname, &time, &packet, &nbytes, &bufsize ) ;
    printf ("starting at pktid #%d = %s at time %s\n", pktid, srcname, s=strtime(time) ) ; 
    free(s) ;

    start_time = now() ;
    while ( totpkts < n2read && ! abort ) {
	    rcode = orbreap ( orb, &pktid, srcname, &time, &packet, &nbytes, &bufsize ) ;
	    if ( rcode < 0 ) {
		complain ( 1, "\norbreap fails\n" ) ; 
		break ;
	    }
	    if ( verbose ) { 
		processPkt ( pktid, srcname, time, packet, nbytes, mode ) ;
	    }

	    totbytes += nbytes ; 
	    totpkts++ ; 
    }

    if ( statefile ) 
	bury() ;

    if ( orbclose(orb) )
	complain ( 1, "Error closing read fd\n" ) ; 
    free(packet) ;

    end_time = now() ;
    delta_t = end_time - start_time ;
    if ( totpkts > 0 ) {
	printf ( "%s: %d %d byte packets (%d kbytes) \n\tin %.3f seconds => %10.3f kbytes/s, %10.3f kbaud, %10.3f pkts/s\n", 
		    Program_Name,
		    totpkts, totbytes/totpkts, totbytes/1024, 
		    delta_t, 
		    totbytes/delta_t/1024,
		    totbytes/delta_t/1024 * 8,
		    totpkts/delta_t ) ; 
    } else { 
	die ( 0, "collected no packets\n" ) ; 
    }

    return 0 ;
}

