#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>

#include "coords.h"
#include "db.h"
#include "stock.h"

typedef struct Flags {
    unsigned int    verbose:4;
}               Flags;

static void
usage ()
{
    cbanner ( "$Date: 2002/05/24 14:42:50 $", 
	     "[-v] database email", 
	     "BRTT example", 
	     "BRTT", 
	     "support@brtt.com" ) ; 
    exit (1);
}

void 
sendmail (Dbptr db, int n, int n2, char *email, Flags flags)
{
    FILE           *file;
    char           *filename,
                    cmd[256];
    int		    result ;
    double          lat,
                    lon,
                    depth,
                    mb;

    filename = tmpnam (0);
    file = fopen (filename, "w");

    for (db.record = n; db.record < n2; db.record++) {
	if (dbgetv (db, 0,
		    "lat", &lat,
		    "lon", &lon,
		    "depth", &depth,
		    "mb", &mb,
		    0)
	    == dbINVALID) {
	    complain (0, "couldn't read record #%d from origin table", db.record);
	} else {
	    fprintf (file, "%5.1f   %8.3f %8.3f  %6.2f\n", mb, lat, lon, depth);
	}
    }

    fclose (file);
    sprintf (cmd, "mailx -s 'new origin' %s < %s", email, filename);
    if ( flags.verbose ) { 
	elog_notify ( 0, cmd ) ; 
    }
    result = system (cmd);
    if ( result ) { 
	complain ( 1, "failed to send email: %s", cmd ) ; 
    }
    unlink (filename);
}

int
main (int argc, char **argv)
{
    int             c,
                    errflg = 0;
    int             n,
                    n2;
    Dbptr           db;
    char           *dbname;
    char           *email;
    Flags           flags;
    Pf             *pf;
    int             sleeptime = 60;

    memset (&flags, 0, sizeof (flags));
    elog_init (argc, argv);

    while ((c = getopt (argc, argv, "s:vV")) != -1) {
	switch (c) {

	  case 's':
	    sleeptime = atoi(optarg) ;
	    break;

	  case 'v':
	    flags.verbose++;
	    break;

	  case 'V':
	    usage() ;
	    exit (0);

	  default:
	    errflg++;

	    break;
	}
    }

    if (errflg || argc-optind != 2)
	usage ();

    dbname = argv[optind++];
    email = argv[optind++];

    dbopen (dbname, "r", &db);
    db = dblookup (db, 0, "origin", 0, 0);
    dbquery (db, dbRECORD_COUNT, &n);
    for (;;) {
	sleep (sleeptime);
	dbquery (db, dbRECORD_COUNT, &n2);
	if (n2 > n) {
	    sendmail (db, n, n2, email, flags);
	    n = n2;
	}
    }
    return 0;
}
