#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>

#include "tr.h"

typedef struct Flags { 
    unsigned int verbose : 4 ; 
    unsigned int display : 1 ;
} Flags ;

static void
usage ()
{
    cbanner ( "$Date: 2002/05/24 14:42:50 $", 
	     "[-d] [-o dbout] [-s subset] [-v] database time endtime",
	     "BRTT example", 
	     "BRTT", 
	     "support@brtt.com" ) ; 
    exit (1);
}

int
main (int argc, char **argv)
{
    int             c,
                    errflg = 0;
    Dbptr           db, tr;
    char           *database;
    Flags	    flags ;
    Pf             *pf;
    Tbl		   *input, *fields ;
    char	   *start, *stop ;
    char	   *output=0, *subset=0 ;
    int		    nrecords ;

    memset ( &flags, 0, sizeof(flags)) ;
    elog_init ( argc, argv ) ; 

    if (pfread (Program_Name, &pf) != 0)
	die (0, "Can't read parameter file\n");

    while ((c = getopt (argc, argv, "do:s:vV")) != -1) {
	switch (c) {

	case 'd':
	    flags.display = 1 ; 
	    break ; 

	case 'o':
	    output = optarg ; 
	    break ;

	case 's':
	    subset = optarg ; 
	    break ;

	case 'v':
	    flags.verbose++ ;
	    break;

	case 'V':
	    usage() ;
	    exit (0);

	default:
	    errflg++;
	    break ;
	}
    }

    if (errflg || argc-optind != 3)
	usage ();

    database = argv[optind++];
    start = argv[optind++];
    stop = argv[optind++];
    if ( dbopen(database, "r", &db) ) { 
        die ( 0, "Can't open database %s", database ) ;
    }

    input = pfget_tbl (pf, "view" ) ;
    db = dbprocess ( db, input, 0 ) ;
    if ( subset ) { 
	db = dbsubset ( db, subset, 0 ) ; 
    }

    tr = dbinvalid() ;
    if ( trload_css ( db, start, stop, &tr, 0, 0 ) ) { 
	die ( 1, "trload_css failed" ) ; 
    }

    /* eliminate marked gaps from waveforms; these
       are gaps where no data was recorded, but special missing values
       were inserted instead of ending the wfdisc record */
    if ( trsplit(tr, 0, 0) ) { 
	complain ( 0, "trsplit failed" ) ; 
    }

    /* splice together any adjacent data segments which can be */
    if ( trsplice(tr, 0, 0, 0) ) { 
	complain ( 0, "trsplice failed" ) ; 
    }

    if ( flags.verbose ) { 
	fields = pfget_tbl ( pf, "fields" ) ;
	dbselect (tr, fields, stdout ) ;
	dbquery ( tr, dbRECORD_COUNT, &nrecords ) ; 
	for ( tr.record = 0 ; tr.record < nrecords ; tr.record++ ) { 
	    int i ; 
	    float *data ;
	    dbgetv(tr, 0, "data", &data, 0 ) ; 
	    for ( i=0 ; i<5 ; i++ ) { 
		printf ( "  %12.3f", data[i] ) ; 
	    }
	    printf ( "\n" ) ; 
	}
    }

    if ( flags.display ) { 
	trdisp (tr, "example display") ;
    }

    if ( output ) { 
	Dbptr dbout ;
	if ( dbopen(output, "r+", &dbout) ) { 
	    die ( 1, "Can't open output database %s", output ) ; 
	}
	dbout = dblookup ( dbout, 0, "wfdisc", 0, 0 ) ;
	if ( trsave_wf ( tr, dbout, "sd", 0, 0 ) ) { 
	    die ( 0, "trsave_wf failed" ) ; 
	}
    }

    tr.table = dbALL ;
    trfree(tr) ;
    pffree(pf) ; 
    return 0;
}
