#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>

#include "coords.h"
#include "db.h"
#include "stock.h"

typedef struct Flags { 
    unsigned int verbose : 4 ; 
} Flags ;

static void
usage ()
{
    cbanner ("$Revision: 1.3 $ $Date: 2002/05/24 14:42:51 $",
	    "[-v] lat lon depth time",
	     "BRTT example", 
	     "BRTT",
	     "support@brtt.com" ) ; 
    exit (1);
}

int
main (int argc, char **argv)
{
    int             c,
                    errflg = 0;
    Dbptr           db;
    int		    verbose = 0;
    Pf             *pf;
    char	   *table, *lat, *lon, *depth ; 
    char	    expr[256], staname[128], *s1, *s2 ; 
    int		    n ; 
    double	    epoch, pred ;
    Tbl		   *keys ; 
    Expression     *pgm ;

    elog_init ( argc, argv ) ; 

    if (pfread ("predict", &pf) != 0)
	die (0, "Can't read parameter file\n");

    while ((c = getopt (argc, argv, "vV")) != -1) {
	switch (c) {

	case 'v':
	    verbose++ ;
	    break;

	case 'V':
	    usage() ;
	    break ;

	default:
	    errflg++;
	    break ;
	}
    }

    if (errflg || argc-optind != 4)
	usage ();

    lat   = argv[optind++];
    lon   = argv[optind++];
    depth = argv[optind++];
    epoch = str2epoch(argv[optind++]);

    table = pfget_string (pf, "table" ) ; 

    if ( dbopen_table ( table, "r", &db ) < 0) { 
	die ( 0, "Can't open table %s\n", table ) ; 
    }
    sprintf ( expr, "distance(lat,lon,%s,%s)", lat, lon ) ; 
    keys = strtbl(expr, 0) ; 
    db = dbsort( db, keys, 0, 0 ) ; 

    sprintf ( expr, "ptime(distance(lat,lon,%s,%s),%s)", lat, lon, depth ) ; 

    dbex_compile(db, expr, &pgm, 0 ) ; 
    dbquery(db, dbRECORD_COUNT, &n ) ; 
    for ( db.record=0 ; db.record<n ; db.record++ ) { 
	dbex_eval (db, pgm, 0, &pred ) ; 
	pred += epoch ; 
	dbgetv(db, 0, "staname", staname, 0 ) ; 
	printf ("%-55s %s   (%s)\n", staname, 
		s1=strtime(pred), 
		s2=zepoch2str(pred, "%H:%M:%S.%s %z", "")) ;
	free(s1) ; 
	free(s2) ; 
    }
    dbex_free(pgm) ; 
    return 0;
}
