#!/bin/sh
if [ $# != 4 ] ; then 
    echo "Usage: $0 lat lon depth time" > /dev/stderr
    exit 1
fi

# matlab spits out garbage on stdout, so write to stderr

exec /usr/local/bin/matlab -nodisplay -nojvm -nosplash << EOF > /dev/null

fid = fopen ( '/dev/stderr', 'w' ) ;

lat = $1 ;
lon = $2 ;
depth = $3 ;
time  = '$4' ;

pf = dbpf ( 'predict' ) ;
dbtable = pfget ( pf, 'table' ) ; 
[db, table] = strtok(dbtable, '.') ; 
table = strrep(table, '.', '') ; 

db = dbopen (db, 'r' ) ;
db = dblookup_table ( db, table)  ;
expr =  sprintf ('distance(lat,lon,%f, %f)', lat, lon) ;
db = dbsort( db, expr ) ;

epoch = str2epoch (time) ;

expr = sprintf ('ptime(distance(lat,lon,%f,%f),%f)', lat, lon, depth ) ;
n = dbquery ( db, 'dbRECORD_COUNT' ) ;
for i = 0:1:n-1
    db.record = i ;
    pred = dbeval(db, expr) + epoch ;  
    staname = dbgetv(db, 'staname' ) ;
    s = strtime(pred) ;
    fprintf ( fid, '%-55s %s\n', staname, s ) ; 
end

exit ;

EOF
